# Copyright (C) 2023 Bootloader.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Bootloader or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Bootloader.
#
# BOOTLOADER MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
# SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
# A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  BOOTLOADER SHALL NOT BE
# LIABLE FOR ANY LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF
# USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Bootloader.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Bootloader or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Bootloader.
#
# BOOTLOADER MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
# SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
# A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  BOOTLOADER SHALL NOT BE
# LIABLE FOR ANY LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF
# USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

import argparse
import sys
from pathlib import Path

from majormode.perseus.constant.logging import LOGGING_LEVEL_LITERAL_STRINGS
from majormode.perseus.constant.logging import LoggingLevelLiteral
from majormode.perseus.utils.logging import DEFAULT_LOGGING_FORMATTER
from majormode.perseus.utils.logging import cast_string_to_logging_level
from majormode.perseus.utils.logging import set_up_logger

from bootloader.ue.utils import uasset_dump


def cast_string_to_path(path: str) -> Path:
    return Path(path)


def parse_arguments() -> argparse.Namespace:
    """
    Convert argument strings to objects and assign them as attributes of
    the namespace.


    :return: An object ``Namespace`` corresponding to the populated
        namespace.
    """
    parser = argparse.ArgumentParser(description="Unreal Engine project's asset list dump")

    parser.add_argument(
        '--logging-level',
        dest='logging_level',
        metavar='LEVEL',
        required=False,
        default=str(LoggingLevelLiteral.info),
        type=cast_string_to_logging_level,
        help=f"specify the logging level ({', '.join(LOGGING_LEVEL_LITERAL_STRINGS)})"
    )

    parser.add_argument(
        '--path',
        dest='path',
        metavar='PATH',
        required=True,
        type=cast_string_to_path,
        help="specify the path to root folder of the Unreal Engine project' assets"
    )

    return parser.parse_args()


def run():
    arguments = parse_arguments()

    set_up_logger(logging_formatter=DEFAULT_LOGGING_FORMATTER, logging_level=arguments.logging_level)

    from timeit import default_timer as timer
    start = timer()
    print(uasset_dump.dump_assets(arguments.path))
    end = timer()
    sys.stderr.write(f"Asset dump completed in {end - start}s")



# Copyright (C) 2023 Bootloader.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Bootloader or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Bootloader.
#
# BOOTLOADER MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
# SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
# A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  BOOTLOADER SHALL NOT BE
# LIABLE FOR ANY LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF
# USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

from majormode.perseus.model.enum import Enum


AssetTypes = Enum(
    'AbilitySystemComponent',
    'Actor',
    'ActorComponent',
    'AIController',
    'AimOffsetBlendSpace',
    'AimOffsetBlendSpace1D',
    'AIPerceptionComponent',
    'AnimationModifier',
    'AnimBlueprint',
    'AnimLayerInterface',
    'AnimComposite',
    'AnimInstance',
    'AnimMontage',
    'AnimSequence',
    'AnimationSharingSetup',
    'ApplicationLifecycleComponent',
    'AtmosphericFog',
    'BehaviorTree',
    'BlackboardData',
    'BlendSpace',
    'BlendSpace1D',
    'BlockingVolume',
    'Blueprint',
    'BlueprintFunctionLibrary',
    'BlueprintGeneratedClass',
    'BlueprintInterface',
    'BlueprintMacroLibrary',
    'BoxReflectionCapture',
    'ButtonStyleAsset',
    'CableActor',
    'CableComponent',
    'CameraActor',
    'CameraAnim',
    'CameraBlockingVolume',
    'Character',
    'CharacterMovementComponent',
    'Class',
    'CullDistanceVolume',
    'CurveBase',
    'DataAsset',
    'DataTable',
    'DecalActor',
    'Default',
    'DefaultPawn',
    'DialogueWave',
    'DialogueVoice',
    'DirectionalLight',
    'DirectionalLightMovable',
    'DirectionalLightStatic',
    'DirectionalLightStationary',
    'DocumentationActor',
    'EditorTutorial',
    'EnvQuery',
    'ExponentialHeightFog',
    'FileMediaSource',
    'Font',
    'FontFace',
    'ForceFeedbackEffect',
    'GameModeBase',
    'GameStateBase',
    'HUD',
    'Interface',
    'InterpData',
    'KillZVolume',
    'Landscape',
    'LandscapeBlueprintBrushBase',
    'LevelBounds',
    'LevelSequence',
    'LevelStreamingVolume',
    'LightmassCharacterIndirectDetailVolume',
    'VolumetricLightmapDensityVolume',
    'LightmassImportanceVolume',
    'MassiveLODOverrideVolume',
    'Material',
    'MaterialFunction',
    'MaterialInstanceActor',
    'MaterialInstanceConstant',
    'MaterialParameterCollection',
    'MediaPlayer',
    'MediaTexture',
    'LevelSequenceActor',
    'MultiFont',
    'NavLinkProxy',
    'NavMeshBoundsVolume',
    'NavModifierComponent',
    'NavModifierVolume',
    'Note',
    'ObjectLibrary',
    'PainCausingVolume',
    'ParticleSystem',
    'Pawn',
    'PawnNoiseEmitterComponent',
    'PawnSensingComponent',
    'PhysicalMaterial',
    'PhysicsAsset',
    'PhysicsConstraintActor',
    'PhysicsConstraintComponent',
    'PhysicsHandleComponent',
    'PhysicsThruster',
    'PhysicsThrusterComponent',
    'PhysicsVolume',
    'PlaneReflectionCapture',
    'PlatformMediaSource',
    'PlayerController',
    'PlayerStart',
    'PointLight',
    'PoseAsset',
    'PostProcessVolume',
    'PrecomputedVisibilityOverrideVolume',
    'PrecomputedVisibilityVolume',
    'ProceduralFoliageVolume',
    'ProceduralFoliageBlockingVolume',
    'ProjectileMovementComponent',
    'RadialForceActor',
    'RadialForceComponent',
    'ReflectionCapture',
    'RotatingMovementComponent',
    'SceneCapture2D',
    'SceneCaptureCube',
    'SceneComponent',
    'SkyLight',
    'SkyLightComponent',
    'SkyAtmosphere',
    'VolumetricCloud',
    'SkeletalMesh',
    'Skeleton',
    'SlateBrushAsset',
    'SlateWidgetStyleAsset',
    'StringTable',
    'SphereReflectionCapture',
    'SpotLight',
    'SpotLightMovable',
    'SpotLightStatic',
    'SpotLightStationary',
    'SpringArmComponent',
    'StaticMesh',
    'StreamMediaSource',
    'SubsurfaceProfile',
    'TargetPoint',
    'TextRenderActor',
    'Texture2D',
    'TextureRenderTarget2D',
    'TextureRenderTargetCube',
    'TriggerBase',
    'TriggerBox',
    'TriggerCapsule',
    'TriggerSphere',
    'TriggerVolume',
    'TouchInterface',
    'UserDefinedCaptureProtocol',
    'UserDefinedEnum',
    'UserDefinedStruct',
    'UserWidget',
    'WidgetBlueprint',
    'WidgetBlueprintGeneratedClass',
    'WindDirectionalSource',
    'World',
    'Cube',
    'Sphere',
    'Cylinder',
    'Cone',
    'Plane',
    'CineCameraActor',
    'CameraRig_Crane',
    'CameraRig_Rail',
    'FoliageType_Actor',
    'FoundationActor',
)



# Copyright (C) 2023 Bootloader.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Bootloader or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Bootloader.
#
# BOOTLOADER MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
# SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
# A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  BOOTLOADER SHALL NOT BE
# LIABLE FOR ANY LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF
# USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

import json
from pathlib import Path

import unreal

from bootloader.ue.model.uasset import UAsset


UNREAL_ENGINE_PROJECT_CONTENT_PATH = '/Game/'


def build_asset(asset_data: unreal.AssetData) -> UAsset:
    """
    Return an instance {@link UAsset} from the asset data.


    :param asset_data: The data of the asset returned by the Unreal Engine
        Python API's Asset Registry.


    :return: An instance {@link UAsset}.
    """
    asset_name = asset_data.asset_name
    package_name = asset_data.package_name

    # @note: The property `asset_class` has been deprecated.  The asset
    #     class name must be converted to full asset pathname.
    asset_class = f'{asset_data.asset_class_path.package_name}/{asset_data.asset_class_path.asset_name}'

    # Find the package names of the assets that this asset depends on.
    dependencies = find_asset_dependencies(asset_data)

    # Build the asset instance.
    asset = UAsset(
        str(asset_name),
        str(asset_class),
        str(package_name),
        dependencies
    )

    return asset


def dump_assets(root_path: str or Path) -> str:
    """
    Serialize the assets found in the specified path to JSON formatted
    string.


    :param root_path: The path to the assets.


    :return: A JSON string representing the list of assets.
    """
    assets = find_assets(root_path)
    json_string = json.dumps([asset.to_json() for asset in assets], indent=2)
    return json_string


def find_asset_dependencies(asset_data: unreal.DataAsset):
    """
    Return the list of the package names of assets that the specified
    asset depends on.

    The function excludes any Unreal Engine's assets.  The function only
    includes assets from the game itself.


    :param asset_data: The data of the asset returned by the Unreal Engine
        Python API's Asset Registry.


    :return: A list of the package names of assets.
    """
    asset_registry = unreal.AssetRegistryHelpers.get_asset_registry()
    dependency_options = unreal.AssetRegistryDependencyOptions(
        # include_soft_package_references=True,
        # include_hard_package_references=True,
        # include_searchable_names=True,
        # include_soft_management_references=True,
        # include_hard_management_references=True,
    )

    asset_dependencies_package_name = asset_registry.get_dependencies(asset_data.package_name, dependency_options)

    return [
        str(package_name)
        for package_name in asset_dependencies_package_name
        if str(package_name).startswith(UNREAL_ENGINE_PROJECT_CONTENT_PATH)
    ]


def find_assets(content_path: str) -> list[UAsset]:
    """
    Return the list of the assets found recursively in the specified path.


    :param content_path: The path to the assets.


    :return: The list of the assets found in the specified path.
    """
    # Retrieve the list of the paths of the assets found recursively in the
    # specified path.
    asset_paths: list = unreal.EditorAssetLibrary.list_assets(content_path, True, False)

    # Build the list of assets.
    #
    # @note: Using list comprehension if faster than `loop` (indeed!):
    #
    #     ```python
    #     assets = []
    #     assets_append = assets.append
    #     for asset_path in asset_paths:
    #         asset_data = unreal.EditorAssetLibrary.find_asset_data(asset_path)
    #         if asset_data.is_valid() and asset_data.is_u_asset():  # @note: Usually always true, but who knows?...
    #             assets_append(build_asset(asset_data))
    #     ```
    #
    #     And faster than `map` when not using a `lambda`:
    #
    #     ```python
    #     assets = list(
    #         filter(
    #             lambda asset: asset is not None,
    #             map(get_asset, asset_paths)
    #         )
    #     )
    #     ```
    #
    #   Result with 1k assets:
    #
    #   - Loop --> 0.6712107429998468
    #   - List comprehension --> 0.05552284200030044
    #   - Map --> 0.20423201099993094
    assets = [
        build_asset(asset_data)
        for asset_data in [
            unreal.EditorAssetLibrary.find_asset_data(asset_path)
            for asset_path in asset_paths
        ]
        if asset_data.is_valid() and asset_data.is_u_asset()
    ]

    return assets


def get_asset(asset_path):
    """
    Return the data of the specified asset.


    :param asset_path: The Unreal Engine path of the asset.


    :return: The asset's data if the asset is valid and is an Unreal
        Engine asset; ``None`` otherwise.
    """
    asset_data = unreal.EditorAssetLibrary.find_asset_data(asset_path)
    if asset_data.is_valid() and asset_data.is_u_asset():
        return build_asset(asset_data)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bootloader',
 'bootloader.ue',
 'bootloader.ue.cli',
 'bootloader.ue.constant',
 'bootloader.ue.model',
 'bootloader.ue.utils']

package_data = \
{'': ['*']}

install_requires = \
['perseus-core-library>=1.19.3,<2.0.0']

entry_points = \
{'console_scripts': ['ueprjver = bootloader.ue.cli.uassetdump:run']}

setup_kwargs = {
    'name': 'uasset-dump',
    'version': '0.0.6',
    'description': 'Command-line Interface (CLI) responsible for returning the list of the assets of an Unreal Engine project into a JSON structure',
    'long_description': '# Unreal Engine Assets Dump\n\nCommand-line Interface (CLI) responsible for returning the list of the assets of an Unreal Engine project into a JSON structure.\n\n## Development\n\n### Poetry\n\n_Unreal Engine Assets Dump_ project used Poetry to declare all its dependencies.  [Poetry](https://python-poetry.org/) is a python dependency management tool to manage dependencies, packages, and libraries in your python project.\n\nWe need to create the Python virtual environment using Poetry:\n\n```shell\npoetry env use /Users/Shared/Epic\\ Games/UE_5.1/Engine/Binaries/ThirdParty/Python3/Mac/bin/python3\n```\n\nWe can enter this virtual environment and install all the required dependencies:\n\n```shell\npoetry shell\npoetry update\n```\n\n\n## Publication\n\nTo publish a new version of the _Unreal Engine Assets Dump_ library to [Pypi](https://pypi.org/), we need to execute the following command:\n\n```shell\npoetry publish --build --username $PYPI_USERNAME --password $PYPI_PASSWORD\n```\n\nWhere the environment variables:\n\n- `$PYPI_USERNAME`: The value `__token__`\n- `$PYPI_PASSWORD`: The [API token](https://pypi.org/manage/account/token/) used to authenticate when uploading packages to PyPI (e.g., `pypi-...`)\n\nWe generally defined a `.env` file and add these environment variables:\n\n```text\n# Copyright (C) 2023 Bootloader.  All rights reserved.\n#\n# This software is the confidential and proprietary information of\n# Bootloader or one of its subsidiaries.  You shall not disclose this\n# confidential information and shall use it only in accordance with the\n# terms of the license agreement or other applicable agreement you\n# entered into with Bootloader.\n#\n# BOOTLOADER MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE\n# SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT\n# NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR\n# A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  BOOTLOADER SHALL NOT BE\n# LIABLE FOR ANY LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF\n# USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.\n\n# The Python Package Index (PyPI) API token to authenticate when\n# uploading this package to PyPI.\nPYPI_USERNAME=__token__\nPYPI_PASSWORD=pypi-(...)\n```\n\n\n## Installation\n\nTo install _Unreal Engine Assets Dump_ library to Unreal Engine, execute the following command.\n\n```shell\n/Users/Shared/Epic\\ Games/UE_5.1/Engine/Binaries/ThirdParty/Python3/Mac/bin/python3 -m pip install --upgrade uasset-dump\n/Users/Shared/Epic\\ Games/UE_5.2/Engine/Binaries/ThirdParty/Python3/Mac/bin/python3 -m pip install --upgrade uasset-dump\n```\n\n/Users/admin/Library/JenkinsAgent/workspace/client-app/scooby-app-build/main/Scooby/Content\n\n## Execution\n\nThen we can load our Unreal Engine project in Unreal Engine Editor and execute the following Python instructions in the Output Log drawer:\n\n```shell\nfrom bootloader.ue.utils import uasset_dump\nuasset_dump.dump_assets(\'/Users/dcaune/Devel/Bootloader/bootloader-human-trainer-ar5/Content\')\n```\n\nThis prints long JSON data.  For instance:\n\n```json\n[\n  {\n    "asset_name": "A_d2_ChangeOutfit_01",\n    "asset_class_path": "/Script/Engine/AnimSequence",\n    "package_name": "/Game/ArtAssets/Pets/QuadrupedDog/Animations/A_d2_ChangeOutfit_01",\n    "dependencies": [\n      "/Game/ArtAssets/Pets/QuadrupedDog/Mesh/S_d2"\n    ]\n  },\n  {\n    "asset_name": "A_d2_ChangeOutfit_02",\n    "asset_class_path": "/Script/Engine/AnimSequence",\n    "package_name": "/Game/ArtAssets/Pets/QuadrupedDog/Animations/A_d2_ChangeOutfit_02",\n    "dependencies": [\n      "/Game/ArtAssets/Pets/QuadrupedDog/Mesh/S_d2"\n    ]\n  },\n  (...)\n]\n```\n\n```shell\nexport UNREAL_ENGINE_PROJECT_PATH=/Users/admin/Library/JenkinsAgent/workspace/client-app/scooby-app-build/main/Scooby/Scooby.uproject\n\n"`find "/Users/Shared/Epic Games/UE_5.2" -name UnrealEditor-Cmd`" \\\n    "$UNREAL_ENGINE_PROJECT_PATH" \\\n    -run=pythonscript \\\n    -stdout \\\n    -Unattended \\\n    -script="/Users/admin/Downloads/run.py"\n\necho $?;\n```',
    'author': 'Daniel CAUNE',
    'author_email': 'daniel@bootloader.studio',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/bootloader-studio/cli-uasset-dump',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



# Copyright (C) 2023 Bootloader.  All rights reserved.
#
# This software is the confidential and proprietary information of
# Bootloader or one of its subsidiaries.  You shall not disclose this
# confidential information and shall use it only in accordance with the
# terms of the license agreement or other applicable agreement you
# entered into with Bootloader.
#
# BOOTLOADER MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
# SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT
# NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
# A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  BOOTLOADER SHALL NOT BE
# LIABLE FOR ANY LOSSES OR DAMAGES SUFFERED BY LICENSEE AS A RESULT OF
# USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.

class UAsset:
    def __init__(
            self,
            asset_name: str,
            asset_class_path: str,
            package_name: str,
            dependencies: list[str] or None):
        """
        Build a new {@link UAsset}.


        :param asset_name: The name of the asset without the package.

        :param asset_class_path: The path name of the asset’s class.

        :param package_name: The name of the package in which the asset is
            found.

        :param dependencies: The list of names of the packages that the asset
            depends on.
        """
        self.__asset_name = asset_name
        self.__asset_class_path = asset_class_path
        self.__package_name = package_name
        self.__dependencies = dependencies

    @property
    def asset_class_path(self) -> str:
        """
        Return the path name of the asset's class.

        Examples:

        ```text
        /Script/Engine/SkeletalMesh
        /Script/Engine/Skeleton
        /Script/Engine/Texture2D
        ```

        :return: The path name of the asset's class.
        """
        return self.__asset_class_path

    @property
    def asset_name(self) -> str:
        """
        Return the name of the asset.


        :return: The name of the asset without the package.
        """
        return self.__asset_name

    @property
    def dependencies(self) -> list[str] or None:
        """
        Return the list of names of the packages that the asset depends on.


        :return: The list of names of the packages that the asset depends on.
        """
        return self.__dependencies

    def to_json(self) -> any:
        return {
            "asset_name": self.__asset_name,
            "asset_class_path": self.__asset_class_path,
            "package_name": self.__package_name,
            "dependencies": self.__dependencies
        }

    @property
    def package_name(self) -> str:
        """
        Return the name of the package in which the asset is found.


        :return: The name of the package in which the asset is found.
        """
        return self.__package_name
